import React from 'react';
import * as TabUtils from './TableauUtils.js';

class TableauVizReact extends React.Component {
constructor(props) {
		super(props);
		this.state = {
			loadingProgress: 'unloaded'
		};
	}

	componentWillMount() {
		this._determineJsApiVersion((jsApiSrc) => {
			const script = document.createElement('script');
			script.onload = function() {
				this._tryInitViz(this.props.url); 
			}.bind(this);

			script.src = jsApiSrc;
			document.body.appendChild(script);
		});
	}

	componentDidMount() {
	}

	componentWillReceiveProps(nextProps) {
		if (nextProps.url != this.props.url) {
			this._tryInitViz(nextProps.url);
		}
	}

	render () {
		return (
				<div id='vizContainer'>
				</div>
		);
	}

  // This is a hack to get around the way the JS API calls document.write to insert the needed library version. Since we need to
	// load the js asynchronously, we can't let the tableau js API do its document.write trick
	_determineJsApiVersion(cb) {
		const parsedUrl = TabUtils.ParseTableauUrl(this.props.url, document);
		const baseJsApiUrl = parsedUrl.server + '/javascripts/api/tableau-2.js';

		// First create an invisible iframe which will load the normal js api. We can't just fetch it directly because CORS
		const iframe = document.createElement('iframe');
		iframe.setAttribute('style', 'visibility:hidden;display:none');
		document.body.appendChild(iframe);

		// Next define the content of the iframe. It includes the baseJsApiUrl and posts a message when it figures out what js version we need
		const content = `<html>
		<script type='text/javascript' src='${baseJsApiUrl}' ></script>
		<script type='text/javascript'>
			var scripts = document.getElementsByTagName('script');
			for (var i in scripts) {
				var src = scripts[i].src;
				if (src && src != '${baseJsApiUrl}') {
					parent.postMessage(src, '${window.location.origin}');
				}
			}
		</script>
		</html>`;

		// This listener will get invoked when the iframe posts a message back up to us
		const childWindowListener = (event) => {
			// We only care about messages which don't have an origin (this means our little iframe we created)
			if (event.origin === 'null') {
				const jsApiUrl = event.data;
				cb(jsApiUrl);

				// Stop listening and remove the iframe
				removeEventListener('message', childWindowListener);
				document.body.removeChild(iframe);
			}
		};

		addEventListener('message', childWindowListener);

		// Finally, set the source of the iframe to initialize the loading
		iframe.src = 'data:text/html;charset=utf-8,' + escape(content);
	}

	_buildVizCreateOptions() {
		let result = {};
		const passThroughProps = ['hideTabs', 'hideToolbar', 'height', 'width', 'device', 'onFirstVizSizeKnown'];
		for (const propName of passThroughProps) {
			if (this.props.hasOwnProperty(propName)) {
				result[propName] = this.props[propName];
			}
		}

		Object.assign(result, this.props.filters);
		result.onFirstInteractive = function(e) {
			this.setState({loadingProgress: 'vizInteractive'});
			if (this.props.onFirstInteractive) {
				this.props.onFirstInteractive(e);
			}
		}.bind(this);

		return result;
	}

	_tryInitViz(urlString) {
		if (this.state.viz) {
			this.state.viz.dispose();
		}

		var containerDiv = document.getElementById('vizContainer');
    const url = TabUtils.ParseTableauUrl(urlString, document).sanitizedUrl;
    const options = this._buildVizCreateOptions();
 
    const viz = new tableau.Viz(containerDiv, url, options);
		this.setState({'viz' : viz});
	}
}

TableauVizReact.prototypes = {
	url: React.PropTypes.string.isRequired,

	// These are mostly the vizCreateOptions list. See https://onlinehelp.tableau.com/current/api/js_api/en-us/JavaScriptAPI/js_api_ref.htm#ref_head_9
	hideTabs: React.PropTypes.bool,
	hideToolbar: React.PropTypes.bool,
	height: React.PropTypes.string,
	width: React.PropTypes.string,
	device: React.PropTypes.string,
	filters: React.PropTypes.object,
	onFirstInteractive: React.PropTypes.func,
	onFirstVizSizeKnown: React.PropTypes.func
};

export default TableauVizReact;
